﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeoSeamDemo
{
    class NewProjection : Projection
    {
        private double rotateAngle;

        Proj4Projection proj4Projection;
        RotationProjection rotateProjection;

        public double Angle
        {
            get { return rotateAngle; }
            set { rotateAngle = value; }
        }

        public NewProjection()
        {
            proj4Projection = new Proj4Projection();
            proj4Projection.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4Projection.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();

            rotateProjection = new RotationProjection();
        }

        protected override void OpenCore()
        {
            if (proj4Projection != null)
            {
                proj4Projection.Open();
            }
            if (rotateProjection != null)
            {
                rotateProjection.Open();
            }
            base.OpenCore();
        }

        protected override void CloseCore()
        {
            if (proj4Projection != null)
            {
                proj4Projection.Close();
            }
            if (rotateProjection != null)
            {
                rotateProjection.Close();
            }
            base.CloseCore();
        }

        protected override Vertex[] ConvertToExternalProjectionCore(double[] x, double[] y)
        {
            //project a layer from Geodetic (EPSG 4326) to Spherical Mercator (Google Map projection)
            Vertex[] projectedVertexs = new Vertex[x.Length];
            for (int i = 0; i < projectedVertexs.Length; i++)
            {
                projectedVertexs[i] = proj4Projection.ConvertToExternalProjection(x[i], y[i]);
            }
            //rotate it
            Vertex[] rotatedVertexs = new Vertex[projectedVertexs.Length];
            rotateProjection.Angle = Angle;
            for (int j = 0; j < projectedVertexs.Length; j++)
            {
                rotatedVertexs[j] = rotateProjection.ConvertToExternalProjection(projectedVertexs[j].X, projectedVertexs[j].Y);
            }

            return rotatedVertexs;
        }

        protected override Vertex[] ConvertToInternalProjectionCore(double[] x, double[] y)
        {
            //project a layer from Geodetic (EPSG 4326) to Spherical Mercator (Google Map projection)
            Vertex[] projectedVertexs = new Vertex[x.Length];
            for (int i = 0; i < projectedVertexs.Length; i++)
            {
                projectedVertexs[i] = proj4Projection.ConvertToInternalProjection(x[i], y[i]);
            }
            //rotate it
            Vertex[] rotatedVertexs = new Vertex[projectedVertexs.Length];
            rotateProjection.Angle = Angle;
            for (int j = 0; j < projectedVertexs.Length; j++)
            {
                rotatedVertexs[j] = rotateProjection.ConvertToInternalProjection(projectedVertexs[j].X, projectedVertexs[j].Y);
            }

            return rotatedVertexs;
        }
    }
}
